#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "unittools.h"

#define  CONVERT     100
#define  SRCUNITNAME "millimeters"
#define  SRCUNIT     "mm"
#define  DSTUNITNAME "decimeters"
#define  DSTUNIT     "dm"

typedef unsigned char uc;

void usage(char *);

int main(int argc, char **argv)
{
    //////////////////////////////////////////////////////////////////
    //
    // Declare variables
    //
    char    string[25];
    char   *delim            = NULL;
    char   *str              = NULL;
    char    space            = ' ';
    char    unitflag         = 0;
    char    verbose          = 0;
    double  value            = 0.0;
    double  source           = 0.0;
    FILE   *inp              = NULL;
    FILE   *out              = stdout;
    uc      precis           = PRECIS;
    int     count            = -1;
    int     opt              = 0;

    //////////////////////////////////////////////////////////////////
    //
    // Argument processing loop
    //
    while ((opt = getopt(argc, argv, "d:hnNp:qvV")) != -1)
    {
        switch (opt)
        {
            case 'd':
                delim        = optarg;
                break;

            case 'h':
            case '?':
                usage(*argv);
                exit(0);

            case 'n':
                unitflag     = 1;
                break;

            case 'N':
                space        = '\b';
                break;

            case 'p':
                sscanf(optarg, "%[0123456789]", string);
                precis       = atoi(string);
                break;

            case 'q':
                out          = fopen("/dev/null", "r");
                break;

            case 'v':
                verbose      = verbose + 1;
                if (verbose >  1)
                    count    = 0;
                break;

            case 'V':
                fprintf(stdout, "%s (%s), version %s\n", *(argv+0), NAME, VERSION);
                exit(0);

            default:
                exit(1);
        }
    }

    //////////////////////////////////////////////////////////////////
    //
    // Determine our source of input, STDIN or file
    //
    if (*(argv+optind)      == NULL)
        inp                  = stdin;
    else
    {
        inp                  = fopen(*(argv+optind), "r");
        if (inp             == NULL)
        {
            fprintf(stderr, "Error opening input file '%s'\n", *(argv+optind));
            exit(1);
        }
    }

    //////////////////////////////////////////////////////////////////
    //
    // If PRECIS environment variable is set, override precision
    //
    str                      = getenv("PRECIS");
    if (str                 != NULL)
        precis               = atoi(str);

    //////////////////////////////////////////////////////////////////
    //
    // Input processing loop - keeping going until EOF is encountered
    //
    opt                      = -2;
    while (!feof(inp))
    {
        //////////////////////////////////////////////////////////////
        //
        // Grab (with input validation) a correct value; and break
        // out of the loop if we encounter an end to input
        //
        opt                  = fscanf(inp, "%[0123456789.]", string);
        if ((opt            == 0)    ||
            (opt            == EOF))
        {
            break;
        }
        fgetc(inp);

        //////////////////////////////////////////////////////////////
        //
        // Obtain source data and convert to destination value
        //
        source               = atof(string);
        value                = source / CONVERT;

        //////////////////////////////////////////////////////////////
        //
        // Display final output, starting with a conditional display
        // of the source data, along with delimiter
        //
        if (verbose         >  0)
        {
            fprintf(out, "%.*lf", precis, source);
            if (unitflag    == 0)
            {
                if (space   == 32)
                    fprintf(out, " ");

                fprintf(out, "%s", SRCUNIT);
            }
            if (delim       == NULL)
                fprintf(out, " -> ");
            else
                fprintf(out, "%s", delim);
        }

        //////////////////////////////////////////////////////////////
        //
        // Display destination value
        //
        fprintf(out, "%.*lf",   precis, value);
        if (unitflag        == 0)
        {
            if (space       == 32)
                fprintf(out, " ");
            fprintf(out, "%s", DSTUNIT);
        }
        fprintf(out, "\n");

        if (count           != -1)
            count            = count + 1;
    }

    //////////////////////////////////////////////////////////////////
    //
    // If enabled, display a count
    //
    if (count               != -1)
    {
        fprintf(out, "Processed %d item", count);
        if (count           != 1)
            fprintf(out, "s");
        fprintf(out, ".\n");
    }

    //////////////////////////////////////////////////////////////////
    //
    // Close custom file pointers
    //
    fclose(inp);
    fclose(out);

    return(0);
}

void usage(char *argv)
{
    fprintf(stdout, "Usage: %s [OPTION]... [FILE]\n", argv);
    fprintf(stdout, "Convert %s (%s) to %s (%s)\n\n", SRCUNITNAME, SRCUNIT, 
                                                      DSTUNITNAME, DSTUNIT);
    fprintf(stdout, "  -d DELIM       set delimiter (default: \" -> \")\n");
    fprintf(stdout, "  -h, -?         display this help and exit\n");
    fprintf(stdout, "  -n             do not display units\n");
    fprintf(stdout, "  -N             no space between value and unit\n");
    fprintf(stdout, "  -p POINTS      set output precision (default: %d)\n", PRECIS);
    fprintf(stdout, "  -q             quiet, do not display anything\n");
    fprintf(stdout, "  -v             verbose, display more information\n");
    fprintf(stdout, "  -V             display version information and exit\n\n");
    fprintf(stdout, "Verbosity compounds; first it will display source data,\n");
    fprintf(stdout, "next it will add a count of items processed.\n");
}
